import Api from '../helpers/api';

class TasksApi extends Api {
  constructor(resource) {
    super(resource);
    this.resource = 'tasks';
  }

  geometries(taskPublicId) {
    let url = `${this.getUrl()}/${taskPublicId}/geometries`;

    const all = async (params) => {
      url = `${url}${this.getUrlParams(params)}`;
      return await this.get(url);
    };

    const attach = async (data) => {
      url = `${url}${this.getUrlParams()}`;
      return await this.post(url, data);
    };

    return {
      all,
      attach,
    };
  }

  users(taskPublicId) {
    let url = `${this.getUrl()}/${taskPublicId}`;

    const attach = async (data) => {
      url = `${url}/users-attach${this.getUrlParams()}`;
      return await this.post(url, data);
    };

    const detach = async (data) => {
      url = `${url}/users-detach${this.getUrlParams()}`;
      return await this.post(url, data);
    };

    return {
      attach,
      detach,
    };
  }

  /**
   * Returns a set of functions for interacting with task categories.
   * @returns {Object} Object containing functions for CRUD operations on task categories.
   */
  categories() {
    /**
     * Generates a URL for task categories based on the provided organization ID and category ID.
     * @param {string} orgId - The ID of the organization.
     * @param {string|null} [categoryId] - The ID of the category (optional).
     * @param {Object|null} [params] - Additional query parameters (optional).
     * @returns {string} The generated URL.
     */
    const generateUrl = (orgId, categoryId, params) => {
      const queryParams = this.getUrlParams(params);

      if (categoryId) {
        return `${this.baseUrl}/task-categories/${categoryId}${queryParams}`;
      } else {
        return `${this.baseUrl}/organizations/${orgId}/task-categories${queryParams}`;
      }
    };

    /**
     * Retrieves all task categories for a given organization.
     * @param {string} orgId - The ID of the organization.
     * @param {Object|null} [params] - Additional query parameters (optional).
     * @returns {Promise} A promise that resolves with the list of task categories.
     */
    const all = async (orgId, params) => {
      const url = generateUrl(orgId, null, params);
      return await this.get(url);
    };

    /**
     * Creates a new task category for a given organization.
     * @param {string} orgId - The ID of the organization.
     * @param {Object} data - The data for creating the task category.
     * @param {Object|null} [params] - Additional query parameters (optional).
     * @returns {Promise} A promise that resolves with the created task category.
     */
    const create = async (orgId, data, params) => {
      const url = generateUrl(orgId, null, params);
      return await this.post(url, data);
    };

    /**
     * Updates a task category.
     * @param {string|null} categoryId - The ID of the category to be updated.
     * @param {Object} data - The data to update the task category with.
     * @param {Object|null} [params] - Additional query parameters (optional).
     * @returns {Promise} A promise that resolves with the updated task category.
     */
    const update = async (categoryId, data, params) => {
      const url = generateUrl(null, categoryId, params);
      return await this.post(url, data, 'patch');
    };

    /**
     * Deletes a task category.
     * @param {string|null} categoryId - The ID of the category to be deleted.
     * @param {Object|null} [params] - Additional query parameters (optional).
     * @returns {Promise} A promise that resolves when the category is deleted.
     */
    const remove = async (categoryId, params) => {
      const url = generateUrl(null, categoryId, params);
      return await this.post(url, null, 'delete');
    };

    return {
      all,
      create,
      update,
      delete: remove,
    };
  }

  evidence(taskPublicId) {
    let url = `${this.getUrl()}/${taskPublicId}/evidence`;

    const all = async (params) => {
      url = `${url}${this.getUrlParams(params)}`;
      return await this.get(url);
    };

    const attach = async (data) => {
      url = `${url}${this.getUrlParams()}`;
      return await this.post(url, data);
    };

    return {
      all,
      attach,
    };
  }
}

export const Tasks = new TasksApi();
