import Api from "../helpers/api";

class TreeCodesApi extends Api {
  constructor(resource) {
    super(resource);
    this.resource = "tree-codes";
  }

  /**
   * @private
   * Constructs the URL for fetching items (campaigns, recipes, etc.) associated with a specific organization, sponsor, or reseller.
   *
   * @param {string} orgType - The shortened identifier for the organization type (o, s, r).
   * @param {string} orgPublicId - The public identifier of the organization.
   * @param {string} item - The specific item (campaigns, recipes, etc.) to fetch.
   * @param {Object} params - Additional parameters for the API request.
   * @returns {string} The constructed URL.
   */
  _constructFetchUrl(orgType, orgPublicId, item, params) {
    return `${this.baseUrl}/${orgType}/${orgPublicId}/${
      this.resource
    }/${item}${this.getUrlParams(params)}`;
  }

  /**
   * @param {string} orgType - Shortened organization type code (o, s, or r).
   * @param {string} orgPublicId - Public ID of the organization.
   * @param {object} params - Optional query parameters.
   * @returns {object} - An object with a method 'all' that, when called, fetches all campaign data.
   */
  campaigns(orgType, orgPublicId, params) {
    /**
     * Fetches campaign data
     */
    const all = async () => {
      const url = this._constructFetchUrl(
        orgType,
        orgPublicId,
        "campaigns",
        params
      );

      return await this.get(url);
    };

    const create = async (data) => {
      const url = this._constructFetchUrl(
        orgType,
        orgPublicId,
        "campaigns",
        params
      );

      return await this.post(url, data);
    };

    const single = async (id, params) => {
      const url = `${this.getUrl()}/campaigns/${id}${this.getUrlParams(params)}`;

      return await this.get(url);
    };

    return {
      all,
      create,
      single,
    };
  }

  recipes() {
    /**
     * Fetches recipes data
     * @param {string} orgType - The shortened identifier for the organization type (o, s, r).
     * @param {string} orgPublicId - The public identifier of the organization.
     * @param {Object} params - Additional parameters for the API request.
     * @returns {Object} An object with a method 'all' to initiate the API request and retrieve recipes based on the provided parameters.
     */
    const all = async (orgType, orgPublicId, params) => {
      const url = this._constructFetchUrl(
        orgType,
        orgPublicId,
        "recipes",
        params
      );

      return await this.get(url);
    };

    const single = async (id, params) => {
      const url = `${this.getUrl()}/recipes/${id}${this.getUrlParams(params)}`;

      return await this.get(url);
    };

    return {
      all,
      single,
    };
  }

  ingredients(recipeId, params) {
    const url = `${this.getUrl()}/recipes/${recipeId}/ingredients${this.getUrlParams(
      params
    )}`;

    const all = async () => {
      return await this.get(url);
    };

    return {
      all,
    };
  }

  batches() {
    /**
     * Fetches batches data
     * @param {string} orgType - The shortened identifier for the organization type (o, s, r).
     * @param {string} orgPublicId - The public identifier of the organization.
     * @param {Object} params - Additional parameters for the API request.
     * @returns {Object} An object with a method 'all' to initiate the API request and retrieve recipes based on the provided parameters.
     */
    const all = async (orgType, orgPublicId, params) => {
      const url = this._constructFetchUrl(
        orgType,
        orgPublicId,
        "batches",
        params
      );

      return await this.get(url);
    };

    const download = async (batchPublicId, params) => {
      return await this.get(`${this.getUrl()}/batches/download/${batchPublicId}${this.getUrlParams(params)}`)
    }

    return {
      all,
      download
    }
  }

  batchByCampaingAndRecipe() {
    /**
     * @param {string} orgType - The shortened identifier for the organization type (o, s, r).
     * @param {string} orgPublicId - The public identifier of the organization.
     * @param {Object} params - Additional parameters for the API request.
     * @returns {Object} An object with a method 'all' to initiate the API request and retrieve recipes based on the provided parameters.
     */
    const post = async (orgType, orgPublicId, params, data) => {
      const url = this._constructFetchUrl(
        orgType,
        orgPublicId,
        "batch-by-campaign-and-recipe",
        params
      );

      return await this.post(url, data);
    }

    return {
      post
    }
  }
}

export const TreeCodes = new TreeCodesApi();
