import { getCookie } from './cookies';
import { createParamsStringFromArgs } from '../utils/args';
import { getSession } from './session';

/**
 * Returns the runtime configuration object for Nuxt 3.
 *
 * Note:
 * This function should only be used with Nuxt 3.
 * For Nuxt 2, the environment variables will work as expected.
 *
 * @returns {Object|null} - The runtime configuration object, or null if it's not available.
 */
// function getNuxtRuntimeConfig() {
//   if (typeof useRuntimeConfig === "function") {
//     return useRuntimeConfig();
//   } else {
//     return null;
//   }
// }

// function getVersion() {
//   if (process) {
//     process?.env?.API_VERITREE_VERSION;
//   } else {
//     const nuxtRuntimeConfig = getNuxtRuntimeConfig();

//     if (nuxtRuntimeConfig) {
//       return nuxtRuntimeConfig.public.API_VERITREE_VERSION;
//     }
//   }

//   return "5.0.0";
// }

/**
 * Adds a version parameter to a URL if it does not already have one.
 * If an environment variable called API_VERITREE_VERSION is defined, its value is used as the version,
 * otherwise the default version "5.0.0" is used.
 *
 * @param {string} url - The URL to modify.
 * @returns {string} The modified URL with the version parameter appended to it.
 */
function addVersionParam(url) {
  // If URL is invalid or already has the result parameter, return it as is
  if (!url || url.includes('_result=1')) {
    return url;
  }

  // Check if URL already has the version parameter
  if (url.includes('_v=')) {
    return url;
  }

  // Use environment variable if available, otherwise use default version
  const version = '25.0.0';

  // Append version parameter to URL
  const urlVersionParam = url.includes('?')
    ? `&_v=${version}`
    : `?_v=${version}`;

  return `${url}${urlVersionParam}`;
}

/**
 * Handles how the data should be sent in the fetch method
 *
 * @param {string} method
 * @param {object} body
 * @returns {object} data
 */
function getConfig(method, data, as) {
  const isGet = method === 'get';
  const isSpoofing = as;
  const isFormData = data instanceof FormData;
  const accessToken = `Bearer ${getCookie('access_token')}`;

  const config = {
    method,
    headers: {
      Authorization: accessToken,
    },
  };

  if (!isFormData) {
    config.headers['Content-Type'] = 'application/json';
  }

  // TODO: improve this ifs and elses
  if (!isGet) {
    if (!data) data = {};

    if (isFormData) {
      if (isSpoofing) data.set('_method', as.toUpperCase());
      config.body = data;
    } else {
      if (isSpoofing) data._method = as.toUpperCase();
      config.body = JSON.stringify(data);
    }
  }

  return config;
}

export default class Api {
  constructor(resource) {
    // Nuxt 3
    // const nuxtConfig = getNuxtRuntimeConfig();
    // const url =
    //   nuxtConfig?.public?.API_VERITREE_URL ||
    //   process?.env?.API_VERITREE_URL ||
    //   null;

    // this.baseUrl = url + "/api";

    this.baseUrl = process.env ? `${process.env.API_VERITREE_URL}/api` : null;
    this.resource = resource ? resource : '';
    this.orgId = null;
    this.orgType = null;
  }

  setBaseUrl(baseUrl) {
    this.baseUrl = baseUrl;
  }

  /**
   *
   * @returns {promise}
   */
  async all(args) {
    const url = `${this.getUrl()}${this.getUrlParams(args)}`;
    return await this.get(url);
  }

  /**
   *
   * @param {string, number} id
   * @param {object} args/params
   * @returns {promise}
   */
  async single(id, args) {
    const url = `${this.getUrl(id)}${this.getUrlParams(args)}`;
    return await this.get(url);
  }

  /**
   *
   * @param {object} data
   * @returns {promise}
   */
  async create(data, args) {
    return await this.post(null, data, null, args);
  }

  
  /**
   * Updates a resource with the specified ID.
   *
   * @param {string} id - The ID of the resource to update.
   * @param {object} data - The data to update the resource with.
   * @param {string} [as='put'] - The HTTP method to use for the update request. Defaults to 'put'.
   * @param {object} args - Additional arguments for the update request.
   * @returns {Promise<any>} - A promise that resolves to the response from the update request.
   */
  async update(id, data, as = 'put', args) {
    const url = `${this.getUrl(id)}${this.getUrlParams(args)}`;
    return await this.post(url, data, as);
  }

  /**
   *
   * @param {string} url
   * @param {object} data
   * @param {string} as - 'put' // necessary for updates because of how Laravel handles PUT requests
   * @returns {promise}
   */
  async delete(id, args) {
    const url = `${this.getUrl(id)}${this.getUrlParams(args)}`;
    return await this.post(url, null, 'delete');
  }

  /**
   *
   * @param {string} url
   * @returns {promise}
   */
  async get(url) {
    return await this.unWrap(url);
  }

  /**
   *
   * @param {string} url
   * @param {object} data
   * @returns {promise}
   */
  async post(url, data, as, args) {
    if (!url) url = `${this.getUrl()}${this.getUrlParams(args)}`;
    return await this.unWrap(url, 'post', data, as);
  }

  // ----------
  // --
  /**
   *
   * @param {string, number} id
   * @returns
   */
  getUrl(id) {
    id = id ? `/${id}` : '';
    return `${this.baseUrl}/${this.resource}${id}`;
  }

  /**
   *
   * @param {object} args
   * @returns {string}
   */
  getUrlParams(args) {
    this.setOrg();
    let isOrgLess = false;
    let isOrgIdAs = false;
    let isOrgTypeAs = false;
    let orgIdParam = '';
    let orgTypeParam = '';
    let argsClone = structuredClone(args); // avoids mutating object

    // while most of endpoints require an org id and type, some endpoints do not
    if (argsClone) {
      isOrgLess = Object.hasOwn(argsClone, 'orgless');
      isOrgIdAs = Object.hasOwn(argsClone, 'org_id_as');
      isOrgTypeAs = Object.hasOwn(argsClone, 'org_type_as');
    }

    // a super admin user can create/edit data from other orgs
    // and for that, we can't use the org id and type in
    // the session.
    if (isOrgIdAs) {
      this.orgId = argsClone.org_id_as;
      delete argsClone.org_id_as;
    }

    if (isOrgTypeAs) {
      this.orgType = argsClone.org_type_as;
      delete argsClone.org_type_as;
    }

    if (!isOrgLess) {
      if (this.orgId) {
        orgIdParam = `org_id=${this.orgId}`;
      }

      if (this.orgType) {
        orgTypeParam = `&org_type=${this.orgType}`;
      }
    }

    return `?${orgIdParam}${orgTypeParam}${createParamsStringFromArgs(
      argsClone
    )}`;
  }

  /**
   * Deals with all fetch requests
   *
   * @param {string} url
   * @param {string} method
   * @param {object} data
   * @returns {object} envelope
   */
  async unWrap(url, method = 'get', data, as) {
    url = addVersionParam(url);
    const config = getConfig(method, data, as);
    const response = await fetch(url, config);

    return new Promise((resolve, reject) => {
      if (response.ok && response.status === 204) {
        resolve();
        return;
      }

      const headerContentType = response.headers.get('content-type');

      if (headerContentType.includes('json')) {
        response.json().then((json) => {
          response.ok ? resolve(json) : reject(json);
        });
      } else if (headerContentType.includes('csv')) {
        response.text().then((res) => {
          response.ok ? resolve(res) : reject(res);
        });
      } else {
        throw new Error('This response header content-type is not handled.');
      }
    });
  }

  setOrg() {
    const session = getSession();
    if (!session) return;
    const { orgId, orgType } = session;
    this.orgId = orgId;
    this.orgType = orgType;
  }
}
