/**
 * Creates a Relations object to handle API relations.
 * @param {object} api - The API object used for making API calls.
 * @param {string} relationPrefix - The prefix for relation endpoints.
 * @param {string} relationId - The identifier for the relation.
 * @returns {object} The Relations object.
 */
const Relations = (api, relationPrefix, relationId) => {
  let url = `${api.baseUrl}/${relationPrefix}/${relationId}/${api.resource}`;

  /**
   * Retrieves all tags.
   * @param {object} params - Additional parameters for the request.
   * @returns {Promise<object>} A promise that resolves with the response data.
   */
  const all = async (params) => {
    url = `${url}${api.getUrlParams(params)}`;
    return await api.get(url);
  };

  /**
   * Retrieves a single tag by ID.
   * @param {string} tagId - The ID of the tag to retrieve.
   * @param {object} params - Additional parameters for the request.
   * @returns {Promise<object>} A promise that resolves with the response data.
   */
  const single = async (tagId, params) => {
    url = `${url}/${tagId}${api.getUrlParams(params)}`;
    return await api.get(url);
  };

  /**
   * Attaches a tag to a resource.
   * @param {string} tagId - The ID of the tag to attach.
   * @param {object} data - The data to send with the request.
   * @param {object} params - Additional parameters for the request.
   * @returns {Promise<object>} A promise that resolves with the response data.
   */
  const attach = async (tagId, data, params) => {
    url = `${url}/${tagId}/attach${api.getUrlParams(params)}`;
    return await api.post(url, data);
  };

  /**
   * Detaches a tag from a resource.
   * @param {string} tagId - The ID of the tag to detach.
   * @param {object} params - Additional parameters for the request.
   * @returns {Promise<object>} A promise that resolves with the response data.
   */
  const detach = async (tagId, params) => {
    url = `${url}/${tagId}/detach${api.getUrlParams(params)}`;
    return await api.post(url, null, "delete");
  };

  /**
   * Retrieves usable tags.
   * @param {object} params - Additional parameters for the request.
   * @returns {Promise<object>} A promise that resolves with the response data.
   */
  const usable = async (params) => {
    url = `${
      api.baseUrl
    }/${relationPrefix}/${relationId}/usable-tags${api.getUrlParams(params)}`;
    return await api.get(url);
  };

  const attached = async (params) => {
    url = `${url}/${api.getUrlParams(params)}`;
    return await api.get(url);
  };

  return {
    all,
    attached,
    single,
    attach,
    detach,
    usable,
  };
};

export default Relations;
